<?php
/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://www.expedy.fr
 * @since             1.0.0
 * @package           Xpdy
 *
 * @wordpress-plugin
 * Plugin Name:       Expedy.fr
 * Plugin URI:        https://www.expedy.fr
 * Description:       Expedy.fr automatise l'impression de vos commandes directement sur vos différentes imprimantes thermiques, vous alerte par SMS etc...
 * Version:           1.0.0
 * Author:            Expedy.fr
 * Author URI:        https://www.expedy.fr
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       xpdy
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Current plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org.
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'XPDY_VERSION', '1.0.0' );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-xpdy-activator.php.
 */
function activate_xpdy() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-xpdy-activator.php';
	Xpdy_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-xpdy-deactivator.php.
 */
function deactivate_xpdy() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-xpdy-deactivator.php';
	Xpdy_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_xpdy' );
register_deactivation_hook( __FILE__, 'deactivate_xpdy' );

/**
 * The core plugin class that is used to define internationalization,.
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-xpdy.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,.
 * then kicking off the plugin from this point in the file does.
 * not affect the page life cycle.
 *
 * @since    1.0.0.
 */
function run_xpdy() {

	$plugin = new Xpdy();
	$plugin->run();

	// Set the plugin slug.
	define( 'XPDY_SLUG', 'wc-settings' );

	// Setting action for plugin.
	add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'xpdy_action_links' );

}
run_xpdy();

/**
 * Adds action link to go to plugin admin.
 *
 * @param array $links an array of admin links.
 *
 * @since    1.0.0.
 */
function xpdy_action_links( $links ) {

	$links[] = '<a href="' . menu_page_url( XPDY_SLUG, false ) . '&tab=integration&section=expedy">Réglages</a>';
	return $links;
}

/**
 * Makes some char clean up.
 *
 * @param string $str to be sanitized.
 *
 * @since    1.0.0.
 */
function strip_accents_now( $str ) {

		$unwanted_array = array(
			'Š' => 'S',
			'š' => 's',
			'Ž' => 'Z',
			'ž' => 'z',
			'À' => 'A',
			'Á' => 'A',
			'Â' => 'A',
			'Ã' => 'A',
			'Ä' => 'A',
			'Å' => 'A',
			'Æ' => 'A',
			'Ç' => 'C',
			'È' => 'E',
			'É' => 'E',
			'Ê' => 'E',
			'Ë' => 'E',
			'Ì' => 'I',
			'Í' => 'I',
			'Î' => 'I',
			'Ï' => 'I',
			'Ñ' => 'N',
			'Ò' => 'O',
			'Ó' => 'O',
			'Ô' => 'O',
			'Õ' => 'O',
			'Ö' => 'O',
			'Ø' => 'O',
			'Ù' => 'U',
			'Ú' => 'U',
			'Û' => 'U',
			'Ü' => 'U',
			'Ý' => 'Y',
			'Þ' => 'B',
			'ß' => 'Ss',
			'à' => 'a',
			'á' => 'a',
			'â' => 'a',
			'ã' => 'a',
			'ä' => 'a',
			'å' => 'a',
			'æ' => 'a',
			'ç' => 'c',
			'è' => 'e',
			'é' => 'e',
			'ê' => 'e',
			'ë' => 'e',
			'ì' => 'i',
			'í' => 'i',
			'î' => 'i',
			'ï' => 'i',
			'ð' => 'o',
			'ñ' => 'n',
			'ò' => 'o',
			'ó' => 'o',
			'ô' => 'o',
			'õ' => 'o',
			'ö' => 'o',
			'ø' => 'o',
			'ù' => 'u',
			'ú' => 'u',
			'û' => 'u',
			'ý' => 'y',
			'þ' => 'b',
			'ÿ' => 'y',
			'\'' => ' ',
			'&euro;' => 'e',
		);
		$str = strtr( $str, $unwanted_array );

			return $str;
}

/**
 * Strips eur symbol, some printers dont like it.
 *
 * @param string $str to be sanitized.
 *
 * @since    1.0.0.
 */
function stripeuro_now( $str ) {

		$unwanted_array = array( '&euro;' => 'e' );
	$str = strtr( $str, $unwanted_array );

			return $str;
}

/**
 * Calculates needed nb white spaces needed for some printout presentation.
 *
 * @param string  $input the actual data for the actual line to be outputed.
 * @param integer $line_width the printer line width in mm.
 * @since    1.0.0.
 */
function get_spacing( $input, $line_width ) {

	$add_some_spaces = '';

	$actual_left_txt_space = $line_width - strlen( $input );

	if ( 0 < $actual_left_txt_space ) {
		$add_some_spaces = str_repeat( ' ', $actual_left_txt_space );
	} else {
		$add_some_spaces = '';}

	return $add_some_spaces;
}

/**
 * Plugin class detection.
 *
 * @since    1.0.0.
 */
if ( ! class_exists( 'CORE_Xpdy' ) ) :
	require plugin_dir_path( __FILE__ ) . 'class-core-xpdy.php';

	$core_xpdy = new CORE_Xpdy( __FILE__ );
endif;


		/**
		 * On paiment ok.
		 *
		 * @param integer $order_id the unique woocommerce order id.
		 * @param string  $print_trigger the print trigger condition value.
		 */
function monsite_woocommerce_payment_complete( $order_id, $print_trigger ) {

	global $woocommerce;
	global $wpdb;
	global $xpdy_time_shift;
	global $xpdy_sid;
	global $xpdy_token;
	global $xpdy_printer_uid;
	global $xpdy_printer_uid_backup;
	global $xpdy_sms_num;

	global $xpdy_printer_size;
	global $xpdy_print_on;
	global $xpdy_logo_url;

	global $xpdy_title;
	global $xpdy_company;
	global $xpdy_company_id;
	global $xpdy_adr1;
	global $xpdy_adr2;
	global $xpdy_zip;
	global $xpdy_city;
	global $xpdy_phone;
	global $xpdy_email;
	global $xpdy_short_opts;
	global $xpdy_print_nb;
	global $xpdy_multistore;

	// $xpdy_multistore = "6:DESNBXP887K||7:DESHKTI4EOW||8:PTDETIOXZQ||";.

	if ( isset( $_SERVER['HTTP_HOST'] ) ) {
		$this_host = esc_url_raw( wp_unslash( $_SERVER['HTTP_HOST'] ) );
	} else {
		$this_host = '';
	}

	if ( 58 == $xpdy_printer_size ) {
		$line_width = 30; // real total char lenght possible is 32.
	} elseif ( 80 == $xpdy_printer_size ) {
		$line_width = 48; // real total char lenght possible is 48.
	} else {
		$line_width = 30;
	}

	if ( $print_trigger == $xpdy_print_on ) {
		// Récupérer la commande.
		$order = wc_get_order( $order_id );

		// Récupérer  le client.
		$user_id = $order->user_id;

		// Récupérer les produits achetés.
		$items = $order->get_items();

		// Faites  ensuite ce que vous voulez !.
		// echo $message;.
		// Par exemple envoyer un email.
		// wp_mail($email, $subject, $message);.

		// Ou encore ajouter des  metas données à votre client.
		// add_user_meta( $user_id, '_nom_meta', $datas_meta, true );.

		$order_data = $order->get_data(); // The Order data.
		$order_shipping_data = $order->get_shipping_method();

		if ( '' != $order->get_meta('wpc_pro_order_time') ) {} else {

				if ($order->get_meta('wpc_pro_order_time') == "Delivery") {
					$order_shipping_data = $order->get_meta('wpc_pro_order_time');
					$delivery_date_pre = str_replace('/', '-', $order->get_meta('wpc_pro_delivery_date'));
					$order_deliverydate = date('d M Y', strtotime($delivery_date_pre));
					$order_deliverytime = $order->get_meta('wpc_pro_delivery_time');
				}

				if ($order->get_meta('wpc_pro_order_time') == "Pickup") {
					$order_shipping_data = $order->get_meta('wpc_pro_order_time');
					$delivery_date_pre = str_replace('/', '-', $order->get_meta('wpc_pro_pickup_date'));
					$order_deliverydate = date('d M Y', strtotime($delivery_date_pre));
					$order_deliverytime = $order->get_meta('wpc_pro_pickup_time');
				}	
		}

		if ( '' != $order->get_meta('ywcdd_order_carrier') ) {
			$delivery_date = str_replace('-', '/', $order->get_meta('ywcdd_order_delivery_date'));
			$delivery_date_final = date('d M Y', strtotime($delivery_date));
			$shipping_date = str_replace('-', '/', $order->get_meta('ywcdd_order_shipping_date'));
			$shipping_date_final = date('d M Y', strtotime($shipping_date));
			$order_shipping_data = $order->get_meta('ywcdd_order_carrier');
			$order_deliverydate = $delivery_date_final."\nLivraison le ".$shipping_date_final;
			$order_deliverytime = $order->get_meta('ywcdd_order_slot_from')."-".$order->get_meta('ywcdd_order_slot_to');
		}

		if ( '' != $order->get_meta( 'pi_delivery_type' ) ) {
			$order_shipping_data = $order->get_meta( 'pi_delivery_type' );
				$order_deliverydate = $order->get_meta( 'pi_delivery_date' );
			  $order_deliverytime = $order->get_meta( 'pi_delivery_time' );
		}

		if ( '' != $order->get_meta( 'byconsolewooodt_delivery_type' ) ) {
			$order_shipping_data = $order->get_meta( 'byconsolewooodt_delivery_type' );
				$order_deliverydate = $order->get_meta( 'byconsolewooodt_delivery_date' );
			  $order_deliverytime = $order->get_meta( 'byconsolewooodt_delivery_time' );
		}

		if ( '' != $order->get_meta( '_local_pickup_time_select' ) ) {
				// $order_shipping_data = $order->get_meta('_local_pickup_time_select');.
				$order_deliverydate = gmdate( 'd/m/Y', $order->get_meta( '_local_pickup_time_select' ) + ( $xpdy_time_shift * 3600 ) );
			  $order_deliverytime = gmdate( 'H:i', $order->get_meta( '_local_pickup_time_select' ) + ( $xpdy_time_shift * 3600 ) );
		}

		if ( '' != $order->get_meta( 'pickup_date' ) ) {
				// $order_shipping_data = $order->get_meta('_local_pickup_time_select');.
				$order_deliverydate = $order->get_meta( 'pickup_date' );
			  $order_deliverytime = $order->get_meta( 'pickup_time' );
		}

		// print_r($order_data);.

		$order_id = $order_data['id'];
		// echo "<br />".$order_id;.

		$order_parent_id = $order_data['parent_id'];
		$order_status = $order_data['status'];
		$order_currency = $order_data['currency'];
		$order_version = $order_data['version'];
		$order_payment_method = $order_data['payment_method'];
		$order_payment_method_title = $order_data['payment_method_title'];
		$order_customer_note = $order_data['customer_note'];
		// echo "<br />".$order_payment_method_title;.

		// Creation and modified WC_DateTime Object date string ##.

		// Using a formated date ( with php date() function as method).
		$order_date_created = $order_data['date_created']->date( 'Y-m-d H:i:s' );
		// echo "<br />".$order_date_created;.
		$order_date_modified = $order_data['date_modified']->date( 'Y-m-d H:i:s' );

		// Using a timestamp ( with php getTimestamp() function as method).
		$order_timestamp_created = $order_data['date_created']->getTimestamp();
		$order_timestamp_modified = $order_data['date_modified']->getTimestamp();

		$order_discount_total = $order_data['discount_total'];
		$order_discount_tax = $order_data['discount_tax'];
		$order_shipping_total = $order_data['shipping_total'];
		$order_shipping_tax = $order_data['shipping_tax'];
		$order_total = $order_data['total'];
		$order_total_tax = $order_data['total_tax'];
		$order_customer_id = $order_data['customer_id']; // ... and so on.

		// BILLING INFORMATION:.

		$order_billing_first_name = $order_data['billing']['first_name'];
		$order_billing_last_name = $order_data['billing']['last_name'];
		// echo "<br />".$order_billing_first_name." ".$order_billing_last_name;.
		$order_billing_company = $order_data['billing']['company'];
		$order_billing_address_1 = $order_data['billing']['address_1'];
		$order_billing_address_2 = $order_data['billing']['address_2'];
		$order_billing_city = $order_data['billing']['city'];
		$order_billing_state = $order_data['billing']['state'];
		$order_billing_postcode = $order_data['billing']['postcode'];
		$order_billing_country = $order_data['billing']['country'];
		$order_billing_email = $order_data['billing']['email'];
		$order_billing_phone = $order_data['billing']['phone'];

		// SHIPPING INFORMATION:.

		$order_shipping_first_name = $order_data['shipping']['first_name'];
		if ( ! $order_shipping_first_name ) {
			$order_shipping_first_name = $order_billing_first_name;}

		$order_shipping_last_name = $order_data['shipping']['last_name'];
		if ( ! $order_shipping_last_name ) {
			$order_shipping_last_name = $order_billing_last_name;}

		$order_shipping_company = $order_data['shipping']['company'];
		if ( ! $order_shipping_company ) {
			$order_shipping_company = $order_billing_company;}

		$order_shipping_address_1 = $order_data['shipping']['address_1'];
		if ( ! $order_shipping_address_1 ) {
			$order_shipping_address_1 = $order_billing_address_1;}

		$order_shipping_address_2 = $order_data['shipping']['address_2'];
		if ( ! $order_shipping_address_2 ) {
			$order_shipping_address_2 = $order_billing_address_2;}

		// echo "<br />".$order_shipping_address_1." ".$order_shipping_address_2;.

		// print_r($order_data['shipping']);.

		$order_shipping_method_title = $order_shipping_data;

		$order_shipping_city = $order_data['shipping']['city'];
		if ( ! $order_shipping_city ) {
			$order_shipping_city = $order_billing_city;}

		$order_shipping_state = $order_data['shipping']['state'];
		if ( ! $order_shipping_state ) {
			$order_shipping_state = $order_billing_state;}

		$order_shipping_postcode = $order_data['shipping']['postcode'];
		if ( ! $order_shipping_postcode ) {
			$order_shipping_postcode = $order_billing_postcode;}

		$order_shipping_country = $order_data['shipping']['country'];
		if ( ! $order_shipping_country ) {
			$order_shipping_country = $order_billing_country;}

		// echo "<br />".$order_shipping_postcode." ".$order_shipping_city." ".$order_shipping_country;.

		// echo "<br /><br />";.
		// Récupérer  le client.

		$cart_details_nice = '';
		// Iterating through each WC_Order_Item_Product objects.

		foreach ( $order->get_items() as $item_key => $item ) {

			// Using WC_Order_Item methods ##.

			// Item ID is directly accessible from the $item_key in the foreach loop or.
			$item_id = $item->get_id();

			if ( '' != $item['_vendor_id'] ) {
					$order_vendor_id = $item['_vendor_id'];
			}

			// added for woo supoption with yith.

			// $woo_db_prefix = $wpdb->prefix;.
			$wooprod_subopt_data = '';

			// START suboption generated by YITH WOOCOMMERCE EXTENSION.

			$wooprod_meta_key_targ = '_ywapo_meta_data';
			$query_wooprod_suboptions_result = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM ' . $wpdb->prefix . 'woocommerce_order_itemmeta WHERE order_item_id = %s AND meta_key = %s ORDER BY meta_id ASC LIMIT 1', $item_id, $wooprod_meta_key_targ ) );

			$meta_value_trg = '';
			foreach ( $query_wooprod_suboptions_result as $key_wooprod => $row_wooprod ) {
				// each column in your row will be accessible like this.
				$meta_value_trg = $row_wooprod->meta_value;
			}

			if ( '' != $meta_value_trg ) {
				$meta_value_trg_ok = unserialize( $meta_value_trg );
				foreach ( $meta_value_trg_ok as $wooprod_suboption ) {
					if ( 0 == $wooprod_suboption['price'] ) {
						$wooprod_suboption_price = '';
					} else {
						$wooprod_suboption_price = ' +' . number_format( $wooprod_suboption['price'], 2, '.', '' ) . 'e';}

					if ( 1 == $xpdy_short_opts ) {
							$wooprod_subopt_data = $wooprod_subopt_data . "\n" . $wooprod_suboption['value'] . $wooprod_suboption_price;
					} else {
						  $wooprod_subopt_data = $wooprod_subopt_data . "\n" . $wooprod_suboption['name'] . ' ' . $wooprod_suboption['value'] . $wooprod_suboption_price;
					}
				}
				$wooprod_subopt_data = $wooprod_subopt_data . "\n";
			}
			// END suboption generated by YITH WOOCOMMERCE EXTENSION.

			// START suboption generated by  TM Extra Product WOOCOMMERCE EXTENSION.

			$wooprod_meta_key_targ = '_tmcartepo_data';
			$query_wooprod_suboptions_result = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM ' . $wpdb->prefix . 'woocommerce_order_itemmeta WHERE order_item_id = %s AND meta_key = %s ORDER BY meta_id ASC LIMIT 1', $item_id, $wooprod_meta_key_targ ) );

			$meta_value_trg = '';
			foreach ( $query_wooprod_suboptions_result as $key_wooprod => $row_wooprod ) {
				// each column in your row will be accessible like this.
				$meta_value_trg = $row_wooprod->meta_value;
			}

			if ( '' != $meta_value_trg ) {
				$meta_value_trg_ok = unserialize( $meta_value_trg );
				foreach ( $meta_value_trg_ok as $wooprod_suboption ) {
					if ( 0 == $wooprod_suboption['price'] ) {
						$wooprod_suboption_price = '';
					} else {
						$wooprod_suboption_price = ' +' . number_format( $wooprod_suboption['price'], 2, '.', '' ) . 'e';}
					if ( 1 == $xpdy_short_opts ) {
							$wooprod_subopt_data = $wooprod_subopt_data . "\n" . $wooprod_suboption['value'] . $wooprod_suboption_price;
					} else {
						  $wooprod_subopt_data = $wooprod_subopt_data . "\n" . $wooprod_suboption['name'] . ' ' . $wooprod_suboption['value'] . $wooprod_suboption_price;
					}
				}
				$wooprod_subopt_data = $wooprod_subopt_data . "\n";
			}
			// END suboption generated by  TM Extra Product WOOCOMMERCE EXTENSION.

			// START suboption generated by  Advanced Product Fields Pro WOOCOMMERCE EXTENSION.

			$wooprod_meta_key_targ = '_wapf_meta';
			$query_wooprod_suboptions_result = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM ' . $wpdb->prefix . 'woocommerce_order_itemmeta WHERE order_item_id = %s AND meta_key = %s ORDER BY meta_id ASC LIMIT 1', $item_id, $wooprod_meta_key_targ ) );

			$meta_value_trg = '';
			foreach ( $query_wooprod_suboptions_result as $key_wooprod => $row_wooprod ) {
				// each column in your row will be accessible like this.
				$meta_value_trg = $row_wooprod->meta_value;
			}

			if ( '' != $meta_value_trg ) {
				$meta_value_trg_ok = unserialize( $meta_value_trg );

				foreach ( $meta_value_trg_ok as $wooprod_suboption ) {

					foreach ( $wooprod_suboption as $wooprod_sub_suboptions ) {

						if ( '' != $wooprod_sub_suboptions['label'] ) {
							if ( 1 == $xpdy_short_opts ) {
								$wooprod_subopt_data = $wooprod_subopt_data . "\n";
							} else {
								$wooprod_subopt_data = $wooprod_subopt_data . "\n" . $wooprod_sub_suboptions['label'] . ': ';
							}
						}

						$ii = 0;
						foreach ( $wooprod_sub_suboptions['values'] as $wooprod_sub_suboption ) {

							if ( 0 == $wooprod_sub_suboption['price'] ) {
								$wooprod_suboption_price = '';
							} else {
								$wooprod_suboption_price = ' +' . number_format( $wooprod_sub_suboption['price'], 2, '.', '' ) . 'e';}

							if ( 0 == $ii ) {
								$wooprod_subopt_data = $wooprod_subopt_data . $wooprod_sub_suboption['label'] . $wooprod_suboption_price;
							} else {
								$wooprod_subopt_data = $wooprod_subopt_data . ', ' . $wooprod_sub_suboption['label'] . $wooprod_suboption_price;

							}

							$ii++;
						}
					}
				}
				$wooprod_subopt_data = $wooprod_subopt_data . "\n";
			}
			// END suboption generated by  Advanced Product Fields Pro WOOCOMMERCE EXTENSION.

			// NO SUBOPTIONS FROM TRADITIONAL SUBOPTION PLUGINS ? .. lets  check if there are some metadata anyway.
			if ( '' == $wooprod_subopt_data ) {

				// $extra_meta_data = $item->get_formatted_meta_data('_', true);.
				$extra_meta_data = $item->get_formatted_meta_data();

				$xtra_ct = 0;
				foreach ( $extra_meta_data as $key_wooprod => $row_wooprod ) {

					if ( 1 == $xpdy_short_opts ) {
						$wooprod_subopt_data = $wooprod_subopt_data . "\n" . stripeuro_now( strip_tags( $row_wooprod->value ) );
					} else {
						$wooprod_subopt_data = $wooprod_subopt_data . "\n" . stripeuro_now( strip_tags( $row_wooprod->key ) ) . ' ' . stripeuro_now( strip_tags( $row_wooprod->value ) );
					}

					$xtra_ct++;

				}
				if ( 0 != $xtra_ct ) {
					$wooprod_subopt_data = $wooprod_subopt_data . "\n";
				}
			}
			// END NO SUBOPTIONS FROM TRADITIONAL SUBOPTION PLUGINS ? .. lets  check if there are some metadata anyway.

			// $product_price  = $item->get_price();.
			$product_price_tax  = $item->get_subtotal();

			// Using WC_Order_Item_Product methods ##.

			$product      = $item->get_product(); // Get the WC_Product object.

			$product_id   = $item->get_product_id(); // the Product id.
			$variation_id = $item->get_variation_id(); // the Variation id.

			$item_type    = $item->get_type(); // Type of the order item ("line_item").

			$item_name    = $item->get_name(); // Name of the product.

			$quantity     = $item->get_quantity();
			$tax_class    = $item->get_tax_class();
			$line_subtotal     = $item->get_subtotal(); // Line subtotal (non discounted).
			$line_subtotal_tax = $item->get_subtotal_tax(); // Line subtotal tax (non discounted).
			$line_total        = $item->get_total(); // Line total (discounted).
			$line_total_tax    = $item->get_total_tax(); // Line total tax (discounted).

			// Access Order Items data properties (in an array of values) ##.
			$item_data    = $item->get_data();

			$product_name = $item_data['name'];

			$product_id   = $item_data['product_id'];
			$variation_id = $item_data['variation_id'];
			$quantity     = $item_data['quantity'];

			$tax_class    = $item_data['tax_class'];
			$line_subtotal     = $item_data['subtotal'];
			$line_subtotal_tax = $item_data['subtotal_tax'];
			$line_total        = $item_data['total'];
			$line_total_tax    = $item_data['total_tax'];

			// Get data from The WC_product object using methods (examples).
			$product        = $item->get_product(); // Get the WC_Product object.

			$product_type   = $product->get_type();
			$product_sku    = $product->get_sku();
			$product_price  = $product->get_price();

			$stock_quantity = $product->get_stock_quantity();

			$actual_left_txt_space = $line_width - strlen( $quantity . ' X ' . strip_accents_now( $product_name ) . ' ' . number_format( $product_price, 2, '.', '' ) . 'e' );

			if ( 0 < $actual_left_txt_space ) {
				$add_some_spaces = str_repeat( ' ', $actual_left_txt_space );
			} else {
				$add_some_spaces = '';}

			$cart_details_nice = $cart_details_nice . $quantity . ' X ' . strip_accents_now( $product_name ) . ' ' . $add_some_spaces . number_format( $product_price, 2, '.', '' ) . 'e' . $wooprod_subopt_data . '
';

		}

		if ( 58 == $xpdy_printer_size ) {
			$data_printer_payload = '
--------------------------------
#  |       Produit      |  Prix 
--------------------------------
';
		} elseif ( 80 == $xpdy_printer_size ) {
			$data_printer_payload = '
------------------------------------------------
#  |              Produit              |  Prix  
------------------------------------------------
';
		} else {
			$data_printer_payload = '
--------------------------------
#  |       Produit      |  Prix 
--------------------------------
';
		}

		$data_printer_payload = $data_printer_payload . $cart_details_nice;

		if ( 58 == $xpdy_printer_size ) {
			$data_printer_payload = $data_printer_payload . '________________________________
';
		} elseif ( 80 == $xpdy_printer_size ) {
			$data_printer_payload = $data_printer_payload . '________________________________________________
';
		} else {
			$data_printer_payload = $data_printer_payload . '________________________________
';
		}

		$add_space = get_spacing( 'TOTAL H.T:' . number_format( floatval( $order_total - $order_total_tax ), 2, '.', '' ) . 'e', $line_width );
		$data_printer_payload = $data_printer_payload . 'TOTAL H.T:' . $add_space . number_format( floatval( $order_total - $order_total_tax ), 2, '.', '' ) . 'e
';

		$add_space = get_spacing( 'TVA:' . number_format( floatval( $order_total_tax ), 2, '.', '' ) . 'e', $line_width );
		$data_printer_payload = $data_printer_payload . 'TVA:' . $add_space . number_format( floatval( $order_total_tax ), 2, '.', '' ) . 'e';

		$add_space = get_spacing( 'TOTAL TTC:' . number_format( floatval( $order_total ), 2, '.', '' ) . 'e', $line_width );
		$data_printer_payload = $data_printer_payload . '<BOLD>TOTAL TTC:' . $add_space . number_format( floatval( $order_total ), 2, '.', '' ) . 'e</BOLD>';

		$add_space = get_spacing( 'LIVRAISON:' . number_format( floatval( $order_shipping_total ), 2, '.', '' ) . 'e', $line_width );
		$data_printer_payload = $data_printer_payload . 'LIVRAISON:' . $add_space . number_format( floatval( $order_shipping_total ), 2, '.', '' ) . 'e
';

		$add_space = get_spacing( 'TOTAL A PAYER:' . number_format( floatval( $order_total ), 2, '.', '' ) . 'e', $line_width );
		$data_printer_payload = $data_printer_payload . '<L>TOTAL À PAYER:' . $add_space . number_format( floatval( $order_total ), 2, '.', '' ) . 'e</L>
';

		if ( 58 == $xpdy_printer_size ) {
			$data_printer_payload = $data_printer_payload . '--------------------------------';
		} elseif ( 80 == $xpdy_printer_size ) {
			$data_printer_payload = $data_printer_payload . '------------------------------------------------';
		} else {
			$data_printer_payload = $data_printer_payload . '--------------------------------';
		}

		$msg_logo = '';
		if ( '' != $xpdy_logo_url ) {
			$msg_logo .= '<IMG>' . $xpdy_logo_url . '</IMG>';
		}

		$msg_entete = '';

		if ( '' != $xpdy_title ) {
			$msg_entete .= '<L><CB>' . $xpdy_title . '</CB></L>';
			$msg_entete = mb_strtoupper( $msg_entete );      // mettre en majuscules.
		}

		$msg_adresse_magasin = '';

		if ( '' != $xpdy_company ) {
			$msg_adresse_magasin .= $xpdy_company . "\n";
		}

		if ( '' != $xpdy_adr1 ) {
			$msg_adresse_magasin .= $xpdy_adr1 . "\n";
		}

		if ( '' != $xpdy_adr2 ) {
			$msg_adresse_magasin .= $xpdy_adr2 . "\n";
		}

		if ( '' != $xpdy_zip ) {
			$msg_adresse_magasin .= $xpdy_zip;

			if ( '' != $xpdy_city ) {
				$msg_adresse_magasin .= ' ';

			}
		}

		if ( '' != $xpdy_city ) {
			 $msg_adresse_magasin .= $xpdy_city;

		}

		if ( '' != $xpdy_city || '' != $xpdy_zip ) {
			$msg_adresse_magasin .= "\n";
		}

		if ( '' != $xpdy_city || '' != $xpdy_zip ) {
			$msg_adresse_magasin .= "\n";
		}

		if ( '' != $xpdy_phone ) {
			$msg_adresse_magasin .= $xpdy_phone . "\n";
		}

		if ( '' != $xpdy_email ) {
			$msg_adresse_magasin .= $xpdy_email . "\n";
		}

		if ( '' != $msg_adresse_magasin ) {
			  $msg_adresse_magasin = '<C>' . $msg_adresse_magasin . '</C>';
			 $msg_adresse_magasin = strip_accents_now( $msg_adresse_magasin ); // supprimer les accents.
			  $msg_adresse_magasin = mb_strtoupper( $msg_adresse_magasin );      // mettre en majuscules.
		}

		$msg_commande_reference_date = '';

		if ( '' != $order_id ) {
			$msg_commande_reference_date .= '<L>Commande ' . $order_id . '</L>';
		}

		if ( '' != $order_date_created ) {
			$msg_commande_reference_date .= gmdate( 'd/m/Y H:i', strtotime( $order_data['date_created'] ) + ( $xpdy_time_shift * 3600 ) ) . "\n";
		}

		// if (isset($order_infos['preferred_date'])) {.
		// $msg_commande_reference_date .= "RETRAIT PREVU LE : ";.
		// $msg_commande_reference_date .= $order_infos['preferred_date']."\nENTRE ".$order_infos['preferred_time']."\n";.
		// }.

		if ( '' != $msg_commande_reference_date ) {
			$msg_commande_reference_date = strip_accents_now( $msg_commande_reference_date ); // supprimer les accents.
			$msg_commande_reference_date = mb_strtoupper( $msg_commande_reference_date );      // mettre en majuscules.
		}

		// Adresse de livraison.

		 $msg_adresse_livraison = '';

		if ( '' != $order_shipping_last_name
			|| '' != $order_shipping_first_name ) {
			$msg_adresse_livraison .= '<BOLD><L>';
		}

		if ( '' != $order_shipping_last_name ) {
			$msg_adresse_livraison .= $order_shipping_last_name;

			if ( '' != $order_shipping_first_name ) {
				$msg_adresse_livraison .= ' ';
			}
		}

		if ( '' != $order_shipping_first_name ) {
			$msg_adresse_livraison .= $order_shipping_first_name;
		}

		if ( '' != $order_shipping_last_name
			|| '' != $order_shipping_first_name ) {
			$msg_adresse_livraison .= "</L></BOLD>\n";
		}

		if ( '' != $order_shipping_address_1 ) {
			$msg_adresse_livraison .= $order_shipping_address_1 . "\n";
		}

		if ( '' != $order_shipping_address_2 ) {
			$msg_adresse_livraison .= $order_shipping_address_2 . "\n";
		}

		if ( '' != $order_shipping_postcode ) {
			$msg_adresse_livraison .= $order_shipping_postcode;

			if ( '' != $order_shipping_city ) {
				$msg_adresse_livraison .= ' ';
			}
		}

		if ( '' != $order_shipping_city ) {
			$msg_adresse_livraison .= $order_shipping_city;
		}

		if ( '' != $order_shipping_postcode
			|| '' != $order_shipping_city ) {
			$msg_adresse_livraison .= "\n";
		}

		if ( '' != $order_billing_phone ) {
			$msg_adresse_livraison .= '<L>' . $order_billing_phone . '</L>';
		}

		if ( '' != $msg_adresse_livraison ) {
			 $msg_adresse_livraison = '<C>' . $msg_adresse_livraison . '</C>';
			 $msg_adresse_livraison = strip_accents_now( $msg_adresse_livraison ); // supprimer les accents.
			 $msg_adresse_livraison = mb_strtoupper( $msg_adresse_livraison );      // mettre en majuscules.
		}

			 // Mode de paiement.

		$msg_mode_paiement = '';

		if ( '' != $order_payment_method ) {
			$msg_mode_paiement .= $order_payment_method_title . ' (' . $order_payment_method . ')';
			$msg_mode_paiement = strip_accents_now( $msg_mode_paiement ); // supprimer les accents.
			$msg_mode_paiement = mb_strtoupper( $msg_mode_paiement );      // mettre en majuscules.
		}

		$msg_delivery_mode = '';

		// Commentaire du client.

		if ( '' != $order_shipping_method_title ) {
			$msg_delivery_mode .= '<BOLD>' . $order_shipping_method_title . '</BOLD>';
			$msg_delivery_mode = strip_accents_now( $msg_delivery_mode ); // supprimer les accents.
			$msg_delivery_mode = mb_strtoupper( $msg_delivery_mode );      // mettre en majuscules.
		}

		if ( '' != $order_deliverydate || '' != $order_deliverytime ) {
			$msg_delivery_mode .= '<BOLD>POUR LE ' . $order_deliverydate . ' - ' . $order_deliverytime . '</BOLD>
';
			$msg_delivery_mode = strip_accents_now( $msg_delivery_mode ); // supprimer les accents.
			$msg_delivery_mode = mb_strtoupper( $msg_delivery_mode );      // mettre en majuscules.
		}

		// Assembler les différentes parties du message.

		$msg = '';

		$msg .= $msg_logo;
		$msg .= $msg_entete;

		if ( '' != $msg_commande_reference_date ) {
			$msg .= "\n" . $msg_commande_reference_date;
		}
		$msg .= $msg_mode_paiement;
		$msg .= $msg_delivery_mode;

		if ( '' != $msg_adresse_livraison ) {
			$msg .= $msg_adresse_livraison;
		}

		$msg .= strip_accents_now( $data_printer_payload );

		if ( '' != $order_customer_note ) {
			$msg .= "\n<C><BOLD><L>NOTE CLIENT</L></BOLD></C>\n" . strip_accents_now( $order_customer_note );
		}

		// $msg .= $msgTotalTTC;.
		// $msg .= $msgTotalHT;.
		// $msg .= $msgTaxesDetail;.
		// if ('' != $msgModeLivraison) $msg .= "\n".$msg_mode_paiement;.

		// $msg .= $msgModeLivraison;.
		if ( '' != $msg_adresse_magasin ) {
			$msg .= "\n" . $msg_adresse_magasin;
		}
		// if ('' != $msgRemerciements)         $msg .= "\n".$msgRemerciements;.

		// découpage du ticket.

			$msg .= '<CUT/>';

		// echo $msg ;.

		// send to print.

		$xpdy_service_url = 'https://www.expedy.fr/api/print';
		$xpdy_service_sms_url = 'https://www.expedy.fr/api/sms';

		$printer_msg = strip_accents_now( $msg );

		// added for multi-vendor.
		$id_store = $order_vendor_id;
		if ( '' != $id_store && '' != $xpdy_multistore ) {
			$search_targ = $id_store;
			$search_found = '';
			$data_string_array = explode( '||', $xpdy_multistore );

			   $pp_nbx = 0;
			foreach ( $data_string_array as $pp_value ) {
				$data_string_subarray = explode( ':', $pp_value );
				$subpp_nbx = 0;

				if ( $search_targ == $data_string_subarray[0] ) {
					$search_found = $data_string_subarray[1];
					break;
				}
				$pp_nbx++;
			}

			if ( '' != $search_found ) {
				$xpdy_printer_uid = $search_found;}
		}

		// end added for multi-vendor.
		if ( '' != $xpdy_printer_uid ) {

			$data_params  = array(
				'printer_uid' => $xpdy_printer_uid,
				'printer_msg' => $printer_msg,
			);

			$data_request = array(
				'sid'    => $xpdy_sid,
				'token'  => $xpdy_token,
				'origin' => $this_host,
				'params' => $data_params,
			);

			// créer et configurer une requête vers l'api.

			$ch = curl_init();
			curl_setopt( $ch, CURLOPT_URL, $xpdy_service_url );
			curl_setopt( $ch, CURLOPT_POST, true );
			curl_setopt( $ch, CURLOPT_POSTFIELDS, http_build_query( $data_request ) );
			curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
			curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, false );
			curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );

			// exécuter la requête pour chaque copie demandée.
			for ( $i = 0; $i < $xpdy_print_nb; $i++ ) {
				$result = curl_exec( $ch );
			}

			curl_close( $ch );
		}

		if ( '' != $xpdy_printer_uid_backup ) {

			$data_params_backup  = array(
				'printer_uid' => $xpdy_printer_uid_backup,
				'printer_msg' => $printer_msg,
			);

			 $data_request_backup = array(
				 'sid'    => $xpdy_sid,
				 'token'  => $xpdy_token,
				 'origin' => $this_host,
				 'params' => $data_params_backup,
			 );

			 // créer et configurer une requête vers l'api.

			 $ch_backup = curl_init();
			 curl_setopt( $ch_backup, CURLOPT_URL, $xpdy_service_url );
			 curl_setopt( $ch_backup, CURLOPT_POST, true );
			 curl_setopt( $ch_backup, CURLOPT_POSTFIELDS, http_build_query( $data_request_backup ) );
			 curl_setopt( $ch_backup, CURLOPT_RETURNTRANSFER, true );
			 curl_setopt( $ch_backup, CURLOPT_SSL_VERIFYHOST, false );
			 curl_setopt( $ch_backup, CURLOPT_SSL_VERIFYPEER, false );

			 // exécuter la requête pour chaque copie demandée.
			 for ( $i_backup = 0; $i_backup < $xpdy_print_nb; $i_backup++ ) {
				 $result_backup = curl_exec( $ch_backup );
			 }

			 curl_close( $ch_backup );

		}

		if ( '' != $xpdy_sms_num ) {

			$sms_sender = 'EXPEDY';

			if ( '' != $this_host ) {
				$sms_msg = 'Nouvelle commande : ' . $order_id . ' (' . number_format( floatval( $order_total ), 2, '.', '' ) . 'EUR) 
-> ' . $this_host . '/wp-admin ';
			} else {
				$sms_msg = 'Nouvelle commande : ' . $order_id . ' (' . number_format( floatval( $order_total ), 2, '.', '' ) . 'EUR) 
         	';
			}
			// end params.

			$data_params_sms = array(
				'sms_sender' => $sms_sender,
				'sms_phone' => $xpdy_sms_num,
				'sms_msg' => $sms_msg,
			);

			// construction requete :.

			$data_request_sms = array(
				'sid' => $xpdy_sid,
				'token' => $xpdy_token,
				'origin' => $this_host,
				'params'  => $data_params_sms,
			);

			// créer et configurer une requête vers l'api.

			$ch_sms = curl_init();
			curl_setopt( $ch_sms, CURLOPT_URL, $xpdy_service_sms_url );
			curl_setopt( $ch_sms, CURLOPT_POST, true );
			curl_setopt( $ch_sms, CURLOPT_POSTFIELDS, http_build_query( $data_request_sms ) );
			curl_setopt( $ch_sms, CURLOPT_RETURNTRANSFER, true );
			curl_setopt( $ch_sms, CURLOPT_SSL_VERIFYHOST, false );
			curl_setopt( $ch_sms, CURLOPT_SSL_VERIFYPEER, false );

			// exécuter la requête pour chaque copie demandée.

			$result_sms = curl_exec( $ch_sms );

			curl_close( $ch_sms );

		}

		// retourner le message.
		// return $msg;.

	}

}


/**
 * Trigger print in this case.
 *
 * @param integer $order_id Unique woocommerce order id.
 * @since    1.0.0.
 */
function print_trigger_paid( $order_id ) {
	monsite_woocommerce_payment_complete( $order_id, 0 );
}


/**
 * Trigger print in this other case.
 *
 * @param integer $order_id Unique woocommerce order id.
 * @since    1.0.0.
 */
function print_trigger_processing( $order_id ) {
	monsite_woocommerce_payment_complete( $order_id, 1 );
}

/**
 * Trigger print in this case likewise.
 *
 * @param integer $order_id Unique woocommerce order id.
 * @since    1.0.0.
 */
function print_trigger_completed( $order_id ) {
	monsite_woocommerce_payment_complete( $order_id, 2 );
}

	add_action( 'woocommerce_payment_complete', 'print_trigger_paid', 10, 1 );

  add_action( 'woocommerce_order_status_processing', 'print_trigger_processing', 10, 1 );

  add_action( 'woocommerce_order_status_completed', 'print_trigger_completed', 10, 1 );
